# Copyright 2016 Canonical Ltd.
# Copyright 2017-18 Telefonica
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
##

from charmhelpers.core.hookenv import (
    action_get,
    action_fail,
    action_set,
    status_set,
    config,
    log
)
from charms.reactive import (
    clear_flag,
    set_flag,
    when,
    when_not,
)
import charms.sshproxy
import sys, traceback
from jinja2 import Environment, FileSystemLoader


@when('sshproxy.configured')
@when_not('mavenir-vepc-conf.installed')
def install_proxy_charm():
    """Post-install actions.
    This function will run when two conditions are met:
    1. The 'sshproxy.configured' state is set
    2. The 'mavenir-vepc-conf.installed' state is not set
    This ensures that the workload status is set to active only when the SSH
    proxy is properly configured.
    """
    set_flag('mavenir-vepc-conf.installed')
    status_set('active', 'Ready!')

@when('actions.touch')
def touch():
    err = ''
    try:
        filename = action_get('filename')
        cmd = ['touch {}'.format(filename)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.touch')

@when('actions.day1-configuration')
def day1Configuration():
    err = ''
    try:
        # The code below is only a placeholder that has to be modified
        # It retrieves the params, and creates a command that will be
        # executed via ssh

        # Step 1. Get params
        mcc = action_get('mcc')
        mnc = action_get('mnc')
        s1_mme_bind_ip_address = action_get('s1_mme_bind_ip_address')
        s1_mme_bind_ip_subnet_mask = action_get('s1_mme_bind_ip_subnet_mask')
        s1_mme_bind_interface = action_get('s1_mme_bind_interface')
        s1u_bind_ip_address = action_get('s1u_bind_ip_address')
        s1u_bind_ip_subnet_mask = action_get('s1u_bind_ip_subnet_mask') 
        s1u_bind_interface_port = action_get('s1u_bind_interface_port')
        sgi_bind_ip_address = action_get('sgi_bind_ip_address')
        sgi_bind_ip_subnet_mask = action_get('sgi_bind_ip_subnet_mask') 
        sgi_bind_interface_port = action_get('sgi_bind_interface_port')
        s6a_mme_host_name = action_get('s6a_mme_host_name')
        s6a_mme_realm_name = action_get('s6a_mme_realm_name')
        s6a_bind_ip_address = action_get('s6a_bind_ip_address')
        s6a_bind_ip_subnet_mask = action_get('s6a_bind_ip_subnet_mask')
        s6a_bind_interface = action_get('s6a_bind_interface')

        #'command {}'.format(ip_pool_name)

        # Step 2. Generate initial_config.cfg from token_files/sample_config_vepc.cfg
        #         properly substituting the params. Suggestion to use jinja2
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_day1_configuration.cfg')
        output_from_parsed_template = template.render (mcc = mcc, 
            mnc = mnc, 
            s1_mme_bind_ip_address = s1_mme_bind_ip_address, 
            s1_mme_bind_ip_subnet_mask = s1_mme_bind_ip_subnet_mask, 
            s1_mme_bind_interface = s1_mme_bind_interface, 
            s1u_bind_ip_address = s1u_bind_ip_address, 
            s1u_bind_ip_subnet_mask = s1u_bind_ip_subnet_mask, 
            s1u_bind_interface_port = s1u_bind_interface_port, 
            sgi_bind_ip_address = sgi_bind_ip_address, 
            sgi_bind_ip_subnet_mask = sgi_bind_ip_subnet_mask, 
            sgi_bind_interface_port = sgi_bind_interface_port,
            s6a_mme_host_name = s6a_mme_host_name,
            s6a_mme_realm_name = s6a_mme_realm_name,
            s6a_bind_ip_address = s6a_bind_ip_address,
            s6a_bind_ip_subnet_mask = s6a_bind_ip_subnet_mask,
            s6a_bind_interface = s6a_bind_interface)

        #print output_from_parsed_template

        # save the results in a temporary file
        temp_file="commands_day1_configuration.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy initial_config.cfg to VEM:/opt/VCM/config/
        # doScp(user,password,host,remotepath,localpath,files)
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("day1-configuration: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.day1-configuration')

@when('actions.new-hss')
def newHSS():
    err = ''
    try:
        # Step 1. Get params
        # new hss
        hss_peer_service_name = action_get('hss_peer_service_name')
        hss_peer_host_name = action_get('hss_peer_host_name')
        hss_host_realm_name = action_get('hss_host_realm_name')
        hss_peer_ip_address = action_get('hss_peer_ip_address')
        hss_group_name = action_get('hss_group_name')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_hss.cfg')
        output_from_parsed_template = template.render (hss_peer_service_name = hss_peer_service_name,
            hss_peer_host_name = hss_peer_host_name,
            hss_host_realm_name = hss_host_realm_name,
            hss_peer_ip_address = hss_peer_ip_address,
            hss_group_name = hss_group_name)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_hss.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-hss: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-hss')

@when('actions.new-ippool')
def newIPPool():
    err = ''
    try:
        # Step 1. Get params
        ip_pool_name  = action_get('ip_pool_name')
        ip_pool_range_start_ip  = action_get('ip_pool_range_start_ip')
        ip_pool_range_end_ip  = action_get('ip_pool_range_end_ip')

        # Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_ippool.cfg')
        output_from_parsed_template = template.render (ip_pool_name  = ip_pool_name,
        ip_pool_range_start_ip  = ip_pool_range_start_ip,
        ip_pool_range_end_ip  = ip_pool_range_end_ip)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_ippool.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")

        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-ippool: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-ippool')


@when('actions.new-pcrf')
def newPCRF():
    err = ''
    try:
        # Step 1. Get params
        # new pcrf
        pgw_pcef_host_name = action_get('pgw_pcef_host_name')
        pgw_pcef_bind_ip_address = action_get('pgw_pcef_bind_ip_address')
        pgw_pcef_bind_ip_subnet_mask = action_get('pgw_pcef_bind_ip_subnet_mask')
        pgw_pcef_bind_interface = action_get('pgw_pcef_bind_interface')
        pgw_pcef_peer_realm_name = action_get('pgw_pcef_peer_realm_name')
        pcrf_peer_host_name = action_get('pcrf_peer_host_name')
        pcrf_peer_realm_name = action_get('pcrf_peer_realm_name')
        pcrf_peer_ip_address = action_get('pcrf_peer_ip_address')
        pcrf_peer_group_name = action_get('pcrf_peer_group_name')
        pcrf_peer_connection_protocol = action_get('pcrf_peer_connection_protocol')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_pcrf.cfg')
        output_from_parsed_template = template.render (pgw_pcef_host_name = pgw_pcef_host_name,
            pgw_pcef_bind_ip_address = pgw_pcef_bind_ip_address,
            pgw_pcef_bind_ip_subnet_mask = pgw_pcef_bind_ip_subnet_mask,
            pgw_pcef_bind_interface = pgw_pcef_bind_interface,
            pgw_pcef_peer_realm_name = pgw_pcef_peer_realm_name,
            pcrf_peer_host_name = pcrf_peer_host_name,
            pcrf_peer_realm_name = pcrf_peer_realm_name,
            pcrf_peer_ip_address = pcrf_peer_ip_address,
            pcrf_peer_group_name = pcrf_peer_group_name,
            pcrf_peer_connection_protocol = pcrf_peer_connection_protocol)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_pcrf.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")

        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-pcrf: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-pcrf')

@when('actions.new-apn')
def newAPN():
    err = ''
    try:
        # Step 1. Get params
        # new apn
        apn_name = action_get('apn_name')
        uplink_apn_ambr = action_get('uplink_apn_ambr')
        downlink_apn_ambr = action_get('downlink_apn_ambr')
        apn_primary_dns_ipv4 = action_get('apn_primary_dns_ipv4')
        apn_secondary_dns_ipv4 = action_get('apn_secondary_dns_ipv4')
        pcrf_peer_group_name = action_get('pcrf_peer_group_name')
        ip_pool_name = action_get('ip_pool_name')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_apn.cfg')
        output_from_parsed_template = template.render (apn_name = apn_name,
            uplink_apn_ambr = uplink_apn_ambr,
            downlink_apn_ambr = downlink_apn_ambr,
            apn_primary_dns_ipv4 = apn_primary_dns_ipv4,
            apn_secondary_dns_ipv4 = apn_secondary_dns_ipv4,
            pcrf_peer_group_name = pcrf_peer_group_name,
            ip_pool_name = ip_pool_name)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_apn.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-apn: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-apn')

@when('actions.new-apn-fqdn')
def newAPNFQDN():
    err = ''
    try:
        # Step 1. Get params
        # new apn
        apn_name = action_get('apn_name')
        mnc = action_get('mnc')
        mcc = action_get('mcc')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_apn_fqdn.cfg')
        output_from_parsed_template = template.render (apn_name = apn_name,
            mnc = mnc,
            mcc = mcc)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_apn_fqdn.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-apn-fqdn: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-apn-fqdn')

@when('actions.new-sgw-service')
def newSGWService():
    err = ''
    try:
        # Step 1. Get params
        # new sgw service
        gw1_S5c_ip_addr = action_get('gw1_S5c_ip_addr')
        gw1_S5c_ip_addr_mask = action_get('gw1_S5c_ip_addr_mask')
        gw1_S5c_interface = action_get('gw1_S5c_interface')
        gw2_s5c_ip_address = action_get('gw2_s5c_ip_address')
        s5u_bind_ip_address = action_get('s5u_bind_ip_address')
        s5u_bind_ip_subnet_mask = action_get('s5u_bind_ip_subnet_mask')
        s5u_bind_interface_port = action_get('s5u_bind_interface_port')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_sgw_service.cfg')
        output_from_parsed_template = template.render (gw1_S5c_ip_addr = gw1_S5c_ip_addr,
            gw1_S5c_ip_addr_mask = gw1_S5c_ip_addr_mask,
            gw1_S5c_interface = gw1_S5c_interface,
            gw2_s5c_ip_address = gw2_s5c_ip_address,
            s5u_bind_ip_address = s5u_bind_ip_address,
            s5u_bind_ip_subnet_mask = s5u_bind_ip_subnet_mask,
            s5u_bind_interface_port = s5u_bind_interface_port)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_sgw_service.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-sgw-service: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-sgw-service')

@when('actions.new-imsi-range')
def newIMSIRange():
    err = ''
    try:
        # Step 1. Get params
        hss_imsi_start_range  = action_get('hss_imsi_start_range')
        hss_imsi_end_range  = action_get('hss_imsi_end_range')
        hss_group_name  = action_get('hss_group_name')

        # Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_new_imsi_range.cfg')
        output_from_parsed_template = template.render (hss_imsi_start_range  = hss_imsi_start_range,
        hss_imsi_end_range  = hss_imsi_end_range,
        hss_group_name  = hss_group_name)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_new_imsi_range.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")

        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("new-imsi-range: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-imsi-range')

@when('actions.modify-plmnid')
def newModifyPLMNId():
    err = ''
    try:
        # Step 1. Get params
        mcc  = action_get('mcc')
        mnc  = action_get('mnc')

        # Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_modify_plmnid.cfg')
        output_from_parsed_template = template.render (mcc  = mcc,
        mnc  = mnc)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_modify_plmnid.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")

        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("modify-plmnid: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.modify-plmnid')

@when('actions.remove-ippool')
def removeIPPool():
    err = ''
    try:
        # Step 1. Get params
        ip_pool_name  = action_get('ip_pool_name')

        # Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_ippool.cfg')
        output_from_parsed_template = template.render (ip_pool_name  = ip_pool_name)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_ippool.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")

        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-ippool: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.remove-ippool')

@when('actions.disable-apn-pcrf')
def disableAPNPCRF():
    err = ''
    try:
        # Step 1. Get params
        # new pcrf
        apn_name = action_get('apn_name')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_disable_apn_pcrf.cfg')
        output_from_parsed_template = template.render (apn_name = apn_name)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_disable_apn_pcrf.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")

        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("disable-apn-pcrf: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.disable-apn-pcrf')

@when('actions.remove-apn')
def removeAPN():
    err = ''
    try:
        # Step 1. Get params
        # new apn
        apn_name = action_get('apn_name')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_apn.cfg')
        output_from_parsed_template = template.render (apn_name = apn_name)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_apn.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-apn: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.remove-apn')

@when('actions.remove-apn-fqdn')
def removeAPNFQDN():
    err = ''
    try:
        # Step 1. Get params
        # new apn
        apn_name = action_get('apn_name')
        mnc = action_get('mnc')
        mcc = action_get('mcc')

       	# Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_apn_fqdn.cfg')
        output_from_parsed_template = template.render (apn_name = apn_name,
            mnc = mnc,
            mcc = mcc)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_apn_fqdn.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")
        
        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-apn-fqdn: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))

    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.new-apn-fqdn')

@when('actions.remove-imsi-range')
def removeIMSIRange():
    err = ''
    try:
        # Step 1. Get params
        hss_imsi_start_range  = action_get('hss_imsi_start_range')
        hss_imsi_end_range  = action_get('hss_imsi_end_range')

        # Step 2. Use jinja to transform template into actual config
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('token_files/sample_remove_imsi_range.cfg')
        output_from_parsed_template = template.render (hss_imsi_start_range  = hss_imsi_start_range,
        hss_imsi_end_range  = hss_imsi_end_range)
        #print output_from_parsed_template
        # save the results in a temporary file
        temp_file="commands_remove_imsi_range.cfg"
        # check that temp_file does not exist
        # TBD
        # Modify temp_file
        with open(temp_file, "w") as fh:
            fh.write(output_from_parsed_template)
            fh.close()

        # Step 3. Copy config.cfg to VEM:/opt/VCM/config/
        # Suggested to use sshproxy lib for that.
        cfg = config()
        host = cfg['ssh-hostname'].split(';')[0]
        user = cfg['ssh-username']
        pwd = cfg['ssh-password']

        charms.sshproxy.sftp(temp_file, "/opt/VCM/config/" + temp_file, host, user, password = pwd)

        log("SFTP OK")

        # Delete local temp_file (the file itself, not the variable)
        # Step 4. Run the configuration command
        cmd = "sudo /opt/VCM/etc/scripts/source_config.sh_vepc " + temp_file + " > /dev/null && echo 'OK'"

        log("Command to execute: " + cmd)

        result, err = charms.sshproxy._run(cmd)

        log("remove-imsi-range: DONE")

    except:
        log("Error. Command Failed.")
        action_fail('command failed:' + err)
        exc_type, exc_value, exc_traceback = sys.exc_info()
        err = traceback.format_exception(exc_type, exc_value, exc_traceback)
        log(str(err))
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.remove-imsi-range')
